@@ -1,23 +1,17 @@
 package Crypt::OpenSSL::X509;
 
 use strict;
-use vars qw($VERSION @EXPORT_OK);
+
 use Exporter;
 use base qw(Exporter);
 
-$VERSION = '1.8';
+our $VERSION = '1.9';
 
-@EXPORT_OK = qw(
-  FORMAT_UNDEF FORMAT_ASN1 FORMAT_TEXT FORMAT_PEM FORMAT_NETSCAPE
+our @EXPORT_OK = qw(
+  FORMAT_UNDEF FORMAT_ASN1 FORMAT_TEXT FORMAT_PEM
   FORMAT_PKCS12 FORMAT_SMIME FORMAT_ENGINE FORMAT_IISSGC OPENSSL_VERSION_NUMBER
 );
 
-sub Crypt::OpenSSL::X509::bit_length {
-  my $x509 = shift;
-
-  return length($x509->modulus) * 4; # each character is one hex digit = 4 bits
-}
-
 sub Crypt::OpenSSL::X509::has_extension_oid {
   my $x509 = shift;
   my $oid  = shift;
@@ -88,14 +82,8 @@ sub Crypt::OpenSSL::X509::is_selfsigned {
   return $x509->subject eq $x509->issuer;
 }
 
-BOOT_XS: {
-  require DynaLoader;
-
-  # DynaLoader calls dl_load_flags as a static method.
-  *dl_load_flags = DynaLoader->can('dl_load_flags');
-
-  do {__PACKAGE__->can('bootstrap') || \&DynaLoader::bootstrap}->(__PACKAGE__, $VERSION);
-}
+use XSLoader;
+XSLoader::load 'Crypt::OpenSSL::X509', $VERSION;
 
 END {
   __PACKAGE__->__X509_cleanup;
@@ -117,16 +105,17 @@ Crypt::OpenSSL::X509 - Perl extension to OpenSSL's X509 API.
 
   print $x509->pubkey() . "\n";
   print $x509->subject() . "\n";
-  print $x509->issuer() . "\n";
-  print $x509->email() . "\n";
   print $x509->hash() . "\n";
+  print $x509->email() . "\n";
+  print $x509->issuer() . "\n";
+  print $x509->issuer_hash() . "\n";
   print $x509->notBefore() . "\n";
   print $x509->notAfter() . "\n";
   print $x509->modulus() . "\n";
   print $x509->exponent() . "\n";
-  print $x509->fingerprint_sha1() . "\n";
   print $x509->fingerprint_md5() . "\n";
-  print $x509->as_string(Crypt::OpenSSL::X509::FORMAT_TEXT) . "\n";
+  print $x509->fingerprint_sha256() . "\n";
+  print $x509->as_string() . "\n";
 
   my $x509 = Crypt::OpenSSL::X509->new_from_string(
     $der_encoded_data, Crypt::OpenSSL::X509::FORMAT_ASN1
@@ -134,9 +123,9 @@ Crypt::OpenSSL::X509 - Perl extension to OpenSSL's X509 API.
 
   # given a time offset of $seconds, will the certificate be valid?
   if ($x509->checkend($seconds)) {
-    # cert is ok at $seconds offset
-  } else {
     # cert is expired at $seconds offset
+  } else {
+    # cert is ok at $seconds offset
   }
 
   my $exts = $x509->extensions_by_oid();
@@ -167,7 +156,7 @@ None by default.
 
 On request:
 
-	FORMAT_UNDEF FORMAT_ASN1 FORMAT_TEXT FORMAT_PEM FORMAT_NETSCAPE
+	FORMAT_UNDEF FORMAT_ASN1 FORMAT_TEXT FORMAT_PEM
 	FORMAT_PKCS12 FORMAT_SMIME FORMAT_ENGINE FORMAT_IISSGC
 
 
@@ -201,12 +190,20 @@ Subject name as a string.
 
 Issuer name as a string.
 
+=item issuer_hash
+
+Issuer name hash as a string.
+
 =item serial
 
 Serial number as a string.
 
 =item hash
 
+Alias for subject_hash
+
+=item subject_hash
+
 Subject name hash as a string.
 
 =item notBefore
@@ -229,6 +226,14 @@ Certificate version as a string.
 
 Signature algorithm name as a string.
 
+=item key_alg_name
+
+Public key algorithm name as a string.
+
+=item curve
+
+Name of the EC curve used in the public key.
+
 =back
 
 =head2 X509 METHODS
@@ -247,11 +252,11 @@ Return Boolean value if subject and issuer name are the same.
 
 =item as_string ( [ FORMAT ] )
 
-Return the certificate as a string in the specified format. C<FORMAT> can be one of C<FORMAT_PEM> (the default), C<FORMAT_ASN1>, or C<FORMAT_NETSCAPE>.
+Return the certificate as a string in the specified format. C<FORMAT> can be one of C<FORMAT_PEM> (the default) or C<FORMAT_ASN1>.
 
 =item modulus ( )
 
-Return the modulus for an RSA public key as a string of hex digits. For DSA, return the public key. Other algorithms are not supported.
+Return the modulus for an RSA public key as a string of hex digits. For DSA and EC return the public key. Other algorithms are not supported.
 
 =item bit_length ( )
 
@@ -261,15 +266,23 @@ Return the length of the modulus as a number of bits.
 
 =item fingerprint_sha1 ( )
 
+=item fingerprint_sha224 ( )
+
+=item fingerprint_sha256 ( )
+
+=item fingerprint_sha384 ( )
+
+=item fingerprint_sha512 ( )
+
 Return the specified message digest for the certificate.
 
 =item checkend( OFFSET )
 
-Given an offset in seconds, will the certificate be expired?
+Given an offset in seconds, will the certificate be expired? Returns True if the certificate will be expired. False otherwise.
 
 =item pubkey ( )
 
-Return the RSA or DSA public key.
+Return the RSA, DSA, or EC public key.
 
 =item num_extensions ( )
 
@@ -281,7 +294,9 @@ Return the Extension specified by the integer C<INDEX>.
 Methods for handling Extension objects are given below.
 
 =item extensions_by_oid ( )
+
 =item extensions_by_name ( )
+
 =item extensions_by_long_name ( )
 
 Return a hash of Extensions indexed by OID or name.
@@ -308,9 +323,11 @@ Methods for handling ObjectID objects are given below.
 
 =item value ( )
 
-Return the value or data of the extension.
-FIXME: the value is returned as a string but may represent
-a complex object.
+Return the value of the extension as an asn1parse(1) style hex dump.
+
+=item as_string ( )
+
+Return a human-readable version of the extension as formatted by X509V3_EXT_print. Note that this will return an empty string for OIDs with unknown ASN.1 encodings.
 
 =back
 
@@ -324,7 +341,7 @@ Return the long name of the object as a string.
 
 =item oid ( )
 
-Return the numeric dot-seperated form of the object identifier as a string.
+Return the numeric dot-separated form of the object identifier as a string.
 
 =back
 
@@ -398,16 +415,27 @@ OpenSSL(1), Crypt::OpenSSL::RSA, Crypt::OpenSSL::Bignum
 
 =head1 AUTHOR
 
-Dan Sully, E<lt>daniel@cpan.orgE<gt>
+Dan Sully
 
 =head1 CONTRIBUTORS
 
-David O'Callaghan, E<lt>david.ocallaghan@cs.tcd.ieE<gt>
-Daniel Kahn Gillmor E<lt>dkg@fifthhorseman.netE<gt>
+=over
+
+=item * Neil Bowers, release 1.8.13
+
+=item * kmx, release 1.8.9
+
+=item * Sebastian Andrzej Siewior
+
+=item * David O'Callaghan, E<lt>david.ocallaghan@cs.tcd.ieE<gt>
+
+=item * Daniel Kahn Gillmor E<lt>dkg@fifthhorseman.netE<gt>
+
+=back
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2004-2011 by Dan Sully
+Copyright 2004-2019 by Dan Sully
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
